package com.sagacloud.parser.simulator;

import com.sagacloud.util.compute_engine.core.GrammarCompare;
import com.sagacloud.util.compute_engine.core.GrammarTreeNode;
import com.sagacloud.util.math.MathUtil;
import lombok.extern.slf4j.Slf4j;
import org.antlr.runtime.ANTLRInputStream;
import org.antlr.runtime.CommonTokenStream;
import org.antlr.runtime.tree.CommonTree;

import java.io.ByteArrayInputStream;
import java.util.ArrayList;
import java.util.List;

@Slf4j
public class String2Grammar {

    public static GrammarTreeNode Generate(String str) throws Exception {
        ANTLRInputStream input = new ANTLRInputStream(new ByteArrayInputStream((str + "$").getBytes()));
        ComputeEngineLexer lexer = new ComputeEngineLexer(input);
        CommonTokenStream tokens = new CommonTokenStream(lexer);
        ComputeEngineParser parser = new ComputeEngineParser(tokens);
        ComputeEngineParser.prog_return r = parser.prog();
        CommonTree t = (CommonTree) r.getTree();

        GrammarTreeNode result = GenerateNode(t);
        return result;
    }

    private static GrammarTreeNode GenerateNode(CommonTree t) throws Exception {
        String text = t.getText();
        List children = t.getChildren();

        if ("if".equals(text) || "elseif".equals(text)) {
            GrammarTreeNode result = new GrammarTreeNode();
            result.type = "compare";
            result.compare = Generate_compare((CommonTree) children.get(0));
            result.compare_yes_node = GenerateNode((CommonTree) children.get(1));
            result.compare_no_node = GenerateNode((CommonTree) children.get(2));
            return result;
        } else if ("else".equals(text)) {
            GrammarTreeNode result = GenerateNode((CommonTree) children.get(0));
            return result;
        } else {
            if (MathUtil.Instance().diy_funname(MathUtil.Instance().fun0p, text)) {
                GrammarTreeNode result = new GrammarTreeNode();
                result.type = "evaluate";
                result.evaluate = text;
                result.evaluateItemList = new ArrayList<GrammarTreeNode>();
                return result;
            } else if (MathUtil.Instance().diy_funname(MathUtil.Instance().fun1p, text)) {
                GrammarTreeNode result = new GrammarTreeNode();
                result.type = "evaluate";
                result.evaluate = text;
                result.evaluateItemList = new ArrayList<GrammarTreeNode>();
                result.evaluateItemList.add(GenerateNode((CommonTree) children.get(0)));
                return result;
            } else if (MathUtil.Instance().diy_funname(MathUtil.Instance().fun2p, text)) {
                GrammarTreeNode result = new GrammarTreeNode();
                result.type = "evaluate";
                result.evaluate = text;
                result.evaluateItemList = new ArrayList<GrammarTreeNode>();
                result.evaluateItemList.add(GenerateNode((CommonTree) children.get(0)));
                result.evaluateItemList.add(GenerateNode((CommonTree) children.get(1)));
                return result;
            } else if ("+".equals(text) || "-".equals(text) || "*".equals(text) || "/".equals(text)) {
                GrammarTreeNode result = new GrammarTreeNode();
                result.type = "evaluate";
                result.evaluate = text;
                result.evaluateItemList = new ArrayList<GrammarTreeNode>();
                result.evaluateItemList.add(GenerateNode((CommonTree) children.get(0)));
                result.evaluateItemList.add(GenerateNode((CommonTree) children.get(1)));
                return result;
            } else if (text.startsWith("other_")) {
                GrammarTreeNode result = new GrammarTreeNode();
                result.type = "other";
                result.other = text.substring("other_".length());
                return result;
            } else {
                try {
                    GrammarTreeNode result = new GrammarTreeNode();
                    result.const_value = Double.parseDouble(text);
                    result.type = "const";
                    return result;
                } catch (NumberFormatException e) {
                    GrammarTreeNode result = new GrammarTreeNode();
                    result.type = "var";
                    result.var_original = text;
                    int index = text.indexOf('\'');
                    if (index != -1) {
                        result.var_sign = text.substring(0, index);
                        result.var_offset = Integer.parseInt(text.substring(index + 1));
                    } else {
                        result.var_sign = text;
                        result.var_offset = 0;
                    }
                    return result;
                }
            }
        }
    }

    private static GrammarCompare Generate_compare(CommonTree t) throws Exception {
        String text = t.getText();
        List children = t.getChildren();

        if (MathUtil.Instance().diy_funname(MathUtil.Instance().cmp, text)) {
            GrammarCompare result = new GrammarCompare();
            result.type = "itom";
            result.itom_op = text;
            result.itom_left = GenerateNode((CommonTree) children.get(0));
            result.itom_right = GenerateNode((CommonTree) children.get(1));
            return result;
        } else if ("&&".equals(text) || "||".equals(text)) {
            GrammarCompare result = new GrammarCompare();
            result.type = "group";
            result.group_op = text;
            result.groupItemList = new ArrayList<GrammarCompare>();
            result.groupItemList.add(Generate_compare((CommonTree) children.get(0)));
            result.groupItemList.add(Generate_compare((CommonTree) children.get(1)));
            return result;
        } else if ("!".equals(text)) {
            GrammarCompare result = new GrammarCompare();
            result.type = "group";
            result.group_op = text;
            result.groupItemList = new ArrayList<GrammarCompare>();
            result.groupItemList.add(Generate_compare((CommonTree) children.get(0)));
            return result;
        } else {
            throw new Exception("GrammarCompare error:" + text);
        }
    }
}
