package com.sagacloud.util.compute_engine.core;


import lombok.extern.slf4j.Slf4j;
import org.reflections.Reflections;

import java.io.File;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.*;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;
@Slf4j
public class CustomFunctionManager {
    private static final CustomFunctionManager instance = new CustomFunctionManager();
    Map<String, ICustomFunction> functionMap = new HashMap<String, ICustomFunction>();

    private CustomFunctionManager() {
        this.LoadClassByReflections();
        this.LoadClassByJar();
    }

    public static CustomFunctionManager Instance() {
        return instance;
    }

    private void LoadClassByReflections() {
        try {
            Reflections reflections = new Reflections("com.sagacloud.extend.customfunction");
            Set<Class<?>> annotated = reflections.getTypesAnnotatedWith(AnnotationDescription.class);
            Iterator<Class<?>> classIter = annotated.iterator();
            while (classIter.hasNext()) {
                Class<?> clazz = classIter.next();
                Class<?>[] interfaces = clazz.getInterfaces();
                boolean flag = false;
                for (int ii = 0; ii < interfaces.length; ii++) {
                    Class<?> inter = interfaces[ii];
                    if ("com.sagacloud.util.compute_engine.core.ICustomFunction".equals(inter.getName())) {
                        flag = true;
                    }
                }
                if (flag) {
                    this.add((ICustomFunction) clazz.newInstance());
                }
            }
        } catch (Exception e) {
            log.error(e.getMessage(),e);
        }
    }

    private void LoadClassByJar() {
        try {
            String dirPath = System.getProperty("user.dir") + "/custom/";
            File dir = new File(dirPath);
            String[] files = new String[0];
            // wrapper
            if (!dir.exists()) {
                dirPath = System.getProperty("user.dir") + "/../lib/custom/";
                dir = new File(dirPath);
                if (dir.exists()) {
                    files = dir.list();
                }
            } else {
                files = dir.list();
            }
            URL[] urls = new URL[files.length];
            for (int i = 0; i < files.length; i++) {
                urls[i] = new URL("file:" + dirPath + files[i]);
            }
            URLClassLoader loader = new URLClassLoader(urls);
            for (int i = 0; i < files.length; i++) {
                File file = new File(dirPath + files[i]);
                if (file.isFile()) {
                    JarFile jar = new JarFile(file);

                    Enumeration<JarEntry> entries = jar.entries();
                    while (entries.hasMoreElements()) {
                        JarEntry entry = entries.nextElement();

                        if (entry.getName().endsWith(".class")) {
                            String name = entry.getName();
                            name = name.substring(0, name.length() - 6);
                            name = name.replaceAll("/", ".");
                            Class<?> clazz = loader.loadClass(name);
                            Class<?>[] interfaces = clazz.getInterfaces();
                            boolean flag = false;
                            for (int ii = 0; ii < interfaces.length; ii++) {
                                Class<?> inter = interfaces[ii];
                                if ("com.sagacloud.util.compute_engine.core.ICustomFunction".equals(inter.getName())) {
                                    flag = true;
                                }
                            }
                            if (flag) {
                                this.add((ICustomFunction) clazz.newInstance());
                            }
                        }
                    }

                    jar.close();
                }
            }
            loader.close();
        } catch (Exception e) {
            log.error(e.getMessage(),e);
        }
    }

    private void add(ICustomFunction function) {
        this.functionMap.put(function.GetName(), function);
    }

    public ICustomFunction Get(String name) {
        return this.functionMap.get(name);
    }

    public double Compute(String custom_function_name, Map<String, String> custom_paramsMap, Map<String, Double> dataMap)
            throws Exception {
        Map<String, Double> inputMap = new HashMap<String, Double>();
        for (String custom_param : custom_paramsMap.keySet()) {
            String var_original = custom_paramsMap.get(custom_param);
            try {
                double const_value = Double.parseDouble(var_original);
                inputMap.put(custom_param, const_value);
            } catch (NumberFormatException e) {
                inputMap.put(custom_param, dataMap.get(var_original));
            }
        }
        ICustomFunction function = this.Get(custom_function_name);
        return function.Compute(inputMap);
    }
}
