package com.sagacloud.util.compute_engine.fjd;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.sagacloud.repository.HBaseSaveManagerRepository;
import com.sagacloud.util.compute_engine.core.*;
import com.sagacloud.util.compute_engine.entity.Entity_ci_instance;
import com.sagacloud.util.compute_engine.entity.Entity_ci_instanceinput;
import com.sagacloud.util.compute_engine.entity.Entity_ci_instanceoutput;
import com.sagacloud.util.compute_engine.entity.Entity_point;
import com.sagacloud.util.compute_engine.util.*;
import com.zillion.database.agent.ZillionAgent;
import com.zillion.util.common.ByteToObject;
import com.zillion.util.zk.ZKLockRW;
import com.zillion.util.zk.ZKWrapper;
import lombok.extern.slf4j.Slf4j;
import org.apache.zookeeper.CreateMode;
import org.apache.zookeeper.data.Stat;
import org.zillion.util.json.FastJsonUtil;

import java.net.InetAddress;
import java.nio.charset.StandardCharsets;
import java.text.SimpleDateFormat;
import java.util.*;

@Slf4j
public class ThreadCompute extends Thread {
    ZillionAgent agent;
    String database_name;
    IDao_log dao_log;

    List<String> children = new ArrayList<String>();
    Random rand = new Random();
    private volatile boolean stop = false;

    public ThreadCompute(String database_name, IDao_log dao_log) {

        this.database_name = database_name;
        this.dao_log = dao_log;
    }

    public void requestStop() {
        stop = true;
    }

    private String Name() {
        try {
            return InetAddress.getLocalHost().getHostAddress() + ":" + this.getName();
        } catch (Exception e) {
           log.error(e.getMessage(),e);
        }
        return this.getName();
    }

    @Override
    public void run() {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");
        ZKWrapper ZK = HBaseSaveManagerRepository.randomZillionAgent().container.ZK;
        int computed_count = 0;
        int child_index = 0;
        String child_name = null;
        try {
            if ("include".equalsIgnoreCase(ConstantCE.config_mode)) {
                children.addAll(ConstantCE.config_instanceList);
            } else {
                if ("exclude".equalsIgnoreCase(ConstantCE.config_mode)) {
                    children = new ArrayList<String>();
                    List<String> children_tmp = ZK.getChildren(ConstantCE.zk_instance, false);
                    for (String child : children_tmp) {
                        boolean exclude = false;
                        for (String config_instance : ConstantCE.config_instanceList) {
                            if (config_instance.equals(child)) {
                                exclude = true;
                                break;
                            }
                        }
                        if (!exclude) {
                            children.add(child);
                        }
                    }
                } else {
                    children = ZK.getChildren(ConstantCE.zk_instance, false);
                }
                Collections.sort(children, new InstanceComparator());
                StringBuffer sb = new StringBuffer();
                for (int i = 0; i < children.size(); i++) {
                    String child = children.get(i);
                    sb.append("\n" + "\t" + i + "\t" + child);
                }
                log.info("ThreadCompute instance initial:" + children.size() + sb);
            }
            if (children.size() > 0) {
                child_index = this.rand.nextInt(children.size());
            }
        } catch (Exception e2) {
            log.error(e2.getMessage(),e2);
        }

        while (!stop) {
            try {
                Thread.sleep(100L);
            } catch (InterruptedException e1) {
                log.error(e1.getMessage(),e1);
            }

            try {
                if ("include".equalsIgnoreCase(ConstantCE.config_mode)) {
                } else {
                    if ("exclude".equalsIgnoreCase(ConstantCE.config_mode)) {
                        children = new ArrayList<String>();
                        List<String> children_tmp = ZK.getChildren(ConstantCE.zk_instance, false);
                        for (String child : children_tmp) {
                            boolean exclude = false;
                            for (String config_instance : ConstantCE.config_instanceList) {
                                if (config_instance.equals(child)) {
                                    exclude = true;
                                    break;
                                }
                            }
                            if (!exclude) {
                                children.add(child);
                            }
                        }
                    } else {
                        children = ZK.getChildren(ConstantCE.zk_instance, false);
                    }
                    Collections.sort(children, new InstanceComparator());
                }
                if (children.size() == 0) {
                    continue;
                }

                if (child_name != null) {
                    for (int i = 0; i < children.size(); i++) {
                        String childInner = children.get(i);
                        if (childInner.equals(child_name)) {
                            child_index = i;
                            break;
                        }
                    }
                }

                child_index++;
                child_index = child_index % children.size();
                child_name = children.get(child_index);
                String name = child_name;
                Stat stat = ZK.exists(ConstantCE.zk_compute_time_last + "/" + name, false);
                if (stat != null) {
                    long compute_interval_seconds;
                    {
                        stat = new Stat();
                        byte[] dataBytes = ZK.getData(ConstantCE.zk_compute_interval + "/" + name, false, stat);
                        String dataString = ByteToObject.toString(dataBytes);
                        compute_interval_seconds = Long.parseLong(dataString);
                    }
                    Date compute_time_last;
                    {
                        stat = new Stat();
                        byte[] dataBytes = ZK.getData(ConstantCE.zk_compute_time_last + "/" + name, false, stat);
                        String dataString = ByteToObject.toString(dataBytes);
                        compute_time_last = sdf.parse(dataString);
                    }
                    Date currentDate = new Date();
                    if (currentDate.getTime() - compute_time_last.getTime() < 1000L * compute_interval_seconds) {
                        continue;
                    }
                }

                String lock_name;
                if (name.startsWith("Pj") || name.startsWith("Bd")) {
                    lock_name = name.substring(0 + 2, 10 + 2);
                } else if (name.startsWith("VOBd")) {
                    lock_name = name.substring(0 + 4, 10 + 4);
                } else {
                    lock_name = name.substring(0, 10);
                }
                String parent_path_meta = ConstantCE.zk_lock + "/" + lock_name;
                stat = ZK.exists(parent_path_meta, false);
                if (stat == null) {
                    try {
                        ZK.create(parent_path_meta, "true".getBytes(StandardCharsets.UTF_8), CreateMode.PERSISTENT);
                    } catch (Exception e) {
                       log.error(e.getMessage(),e);
                    }
                }

                String lock_type_meta = "w";
                // 获取锁
                String node_path_meta = ZKLockRW.Create(ZK, parent_path_meta, lock_type_meta, this.Name());
                try {
                    boolean wait_success = ZKLockRW.WaitOnce(ZK, parent_path_meta, node_path_meta);
                    log.debug("ThreadCompute instance wait lock " + child_name + " " + node_path_meta + " " + wait_success);
                    if (wait_success) {
                        Date currentDate = new Date();
                        boolean computed = this.Compute(name);
                        if (computed) {
                            computed_count++;

                            stat = ZK.exists(ConstantCE.zk_compute_time_last + "/" + name, false);
                            if (stat != null) {
                                ZK.setData(ConstantCE.zk_compute_time_last + "/" + name, sdf.format(currentDate).getBytes(), -1);
                            } else {
                                ZK.create(ConstantCE.zk_compute_time_last + "/" + name, sdf.format(currentDate).getBytes(), CreateMode.PERSISTENT);
                            }
                        }
                    }
                } catch (Exception e) {
                    throw e;
                } finally {
                    // 释放锁
                    ZKLockRW.Finish(ZK, node_path_meta);
                    log.debug("ThreadCompute instance finish lock " + child_name + " " + node_path_meta);
                }
            } catch (Exception e) {
                log.warn(e.getMessage(), e);
            }
        }

        log.warn(this.Name() + "\t" + "computed_count:" + computed_count + "\t" + "finish");
        computed_count = 0;
    }

    private boolean Compute(String name) throws Exception {
        ZKWrapper ZK = HBaseSaveManagerRepository.randomZillionAgent().container.ZK;

        String request;
        {
            String parent_path_meta = ConstantCE.zk_status_request + "/" + name;
            Stat stat = new Stat();
            byte[] data_bytes = ZK.getData(parent_path_meta, false, stat);
            request = new String(data_bytes, StandardCharsets.UTF_8);
        }
        if ("stop".equals(request)) {
            return false;
        }

        int index_ = name.indexOf("_");
        String object_sign = name.substring(0, index_);
        String instance_name = name.substring(index_ + 1);
        Entity_ci_instance ci_instance = InstanceUtil.Get(ConstantCE.database_name, object_sign, instance_name);
        if ("servicedata_stat".equals(instance_name) || "servicedataoftenant_stat".equals(instance_name) || "EMS_stat".equals(instance_name)
                || "tenant_stat".equals(instance_name) || "stat_Inst_stat".equals(instance_name)) {
            ci_instance = InstanceUtil.Get(ConstantCE.database_name, object_sign,
                    instance_name.substring(0, instance_name.length() - "_stat".length()));
            ci_instance.setInstance_name(instance_name);
        } else if ("servicedata_max".equals(instance_name) || "servicedataoftenant_max".equals(instance_name) || "EMS_max".equals(instance_name)
                || "tenant_max".equals(instance_name)) {
            String source_instance_name = instance_name.substring(0, instance_name.length() - "_max".length());
            long compute_interval_seconds = ci_instance.getCompute_interval_seconds();
            ci_instance = InstanceUtil.Get(ConstantCE.database_name, object_sign, source_instance_name);
            ci_instance.setInstance_name(instance_name);
            ci_instance.setCompute_interval_seconds(compute_interval_seconds);

            List<Entity_ci_instanceinput> inputList = new ArrayList<Entity_ci_instanceinput>();
            List<Entity_ci_instanceoutput> outputList = new ArrayList<Entity_ci_instanceoutput>();

            Entity_ci_instanceoutput[] outputs_tmp = ci_instance.getOutputs();
            for (Entity_ci_instanceoutput output_tmp : outputs_tmp) {
                Entity_ci_instanceinput input = new Entity_ci_instanceinput();
                input.setObject_sign(object_sign);
                input.setInstance_name(instance_name);
                input.setInput_sign(output_tmp.getOutput_sign());
                input.setTime_period(output_tmp.getTime_period());
                input.setData_class(output_tmp.getData_class());
                input.setDao_type(output_tmp.getDao_type());
                input.setDao_key(output_tmp.getDao_key());
                input.setSource_object_sign(object_sign);
                input.setSource_instance_name(source_instance_name);
                input.setSource_output_sign(output_tmp.getOutput_sign());
                inputList.add(input);

                Entity_ci_instanceoutput output = new Entity_ci_instanceoutput();
                output.setObject_sign(object_sign);
                output.setInstance_name(instance_name);
                output.setOutput_sign(output_tmp.getOutput_sign());
                output.setTime_begin(output_tmp.getTime_begin());
                output.setTime_end(output_tmp.getTime_end());
                output.setData_class(output_tmp.getData_class());
                output.setDao_type(output_tmp.getDao_type() + "_max");
                output.setDao_key(output_tmp.getDao_key());
                output.setNeed_all_var_data(false);
                output.setTime_period_intialized(true);
                output.setTime_period(output_tmp.getTime_period());
                output.setMode("expression");
                output.setExpression(output_tmp.getOutput_sign());
                outputList.add(output);
            }

            ci_instance.setInputs(inputList.toArray(new Entity_ci_instanceinput[0]));
            ci_instance.setOutputs(outputList.toArray(new Entity_ci_instanceoutput[0]));
        } else if (instance_name.startsWith("fjd_1")) {
            ci_instance = InstanceUtil.Get(ConstantCE.database_name, object_sign, "fjd_0" + instance_name.substring("fjd_0".length()));
            ci_instance.setInstance_name(instance_name);
        }
        if (ci_instance == null) {
            log.error(name + " not exist in db!");
            throw new Exception(name + " not exist in db!");
        }
        if (ci_instance.getFlag_valid() == 0) {
            return false;
        }

        // 增加servicedata_stat、EMS_stat、stat_Inst_stat
        if (ConstantCE.Stat_enable && ("servicedata".equals(instance_name) || "servicedataoftenant".equals(instance_name)
                || "EMS".equals(instance_name) || "tenant".equals(instance_name) || "stat_Inst".equals(instance_name))) {
            String instance_name_tmp = instance_name + "_stat";
            InstanceUtil.Add_tmp(children, object_sign, instance_name, instance_name_tmp, ZK, agent, database_name);
        }
        if (ConstantCE.fjd_1_enable && instance_name.startsWith("fjd_0")) {
            String instance_name_tmp = "fjd_1" + instance_name.substring("fjd_0".length());
            InstanceUtil.Add_tmp(children, object_sign, instance_name, instance_name_tmp, ZK, agent, database_name);
        }

        CoreInstance instance = CoreInstance.Convert(ci_instance);
        String building = instance.object_sign;
        if (instance.instance_name.startsWith("fjd_0") || instance.instance_name.startsWith("fjd_1")) {
            instance.inputList.clear();
            instance.outputList.clear();
            List<String> data_typeList = new ArrayList<String>();
            if (instance.instance_name.contains("_" + "Acc")) {
                data_typeList.add("Acc");
                data_typeList.add("InAc");
            }
            if (instance.instance_name.contains("_" + "Inst")) {
                data_typeList.add("Inst");
            }
            if (instance.instance_name.contains("_" + "EnumV")) {
                data_typeList.add("Boolv");
                data_typeList.add("Enumv");
                data_typeList.add("Step");
                data_typeList.add("Set_Inst");
                data_typeList.add("Set_Boolv");
                data_typeList.add("Set_Enumv");
            }
            if (!QueryUtil.table_exist(database_name, "dy_pointlist")) {
                return false;
            }
            JSONObject Criteria = new JSONObject();
            Criteria.put("building", building);
            JSONArray content = new JSONArray();
            {
                JSONArray contentInner = QueryUtil.select(database_name, "dy_pointlist", Criteria);
                for (int i = 0; i < contentInner.size(); i++) {
                    JSONObject contentItem = (JSONObject) contentInner.get(i);
                    contentItem.put("point_class", "ori");
                    content.add(contentItem);
                }
            }
            for (int i = 0; i < content.size(); i++) {
                JSONObject json = (JSONObject) content.get(i);
                Entity_point detail = (Entity_point) FastJsonUtil.To_JavaObject(json, Entity_point.class);
                boolean match = false;
                for (String data_type : data_typeList) {
                    if (detail.data_type.equalsIgnoreCase(data_type)) {
                        match = true;
                        break;
                    }
                }
                if (!match) {
                    continue;
                }
                if ((ConstantCE.funcid_IncludesMap.size() > 0 && !ConstantCE.funcid_IncludesMap.containsKey(detail.funcid))
                        || ConstantCE.funcid_ExcludesMap.containsKey(detail.funcid)) {
                    continue;
                }
                {
                    Entity_ci_instanceinput input = new Entity_ci_instanceinput();
                    input.setObject_sign(object_sign);
                    input.setInstance_name(instance_name);
                    input.setInput_sign("ori_" + detail.meter + "_" + detail.funcid);
                    input.setTime_period(detail.collect_cycle);
                    input.setData_class("near");
                    input.setDao_type(detail.point_class);
                    JSONObject dao_key = new JSONObject();
                    dao_key.put("building", building);
                    dao_key.put("meter", detail.meter);
                    dao_key.put("funcid", detail.funcid);
                    input.setDao_key(dao_key.toJSONString());
                    input.setSource_object_sign(building);
                    input.setSource_instance_name("ori");
                    input.setSource_output_sign(detail.data_type + "_" + "ori_" + detail.meter + "_" + detail.funcid);
                    CoreInput CoreInput = CoreInstance.Convert_input(input);
                    instance.inputList.add(CoreInput);
                }
                {
                    Entity_ci_instanceoutput output = new Entity_ci_instanceoutput();
                    output.setObject_sign(object_sign);
                    output.setInstance_name(instance_name);
                    output.setOutput_sign("fjd_0_" + detail.meter + "_" + detail.funcid);
                    output.setTime_begin("19700101000000");
                    output.setTime_end("99990101000000");
                    output.setData_class("near");
                    output.setDao_type("fjd_0");
                    JSONObject dao_key = new JSONObject();
                    dao_key.put("building", building);
                    dao_key.put("meter", detail.meter);
                    dao_key.put("funcid", detail.funcid);
                    output.setDao_key(dao_key.toJSONString());
                    output.setNeed_all_var_data(true);
                    output.setTime_period_intialized(true);
                    output.setTime_period(detail.collect_cycle);
                    output.setMode("expression");
                    output.setExpression("ori_" + detail.meter + "_" + detail.funcid);
                    CoreOutput CoreOutput = CoreInstance.Convert_output(output);
                    instance.outputList.add(CoreOutput);
                }
            }
        } else if (instance.instance_name.startsWith("stat_Inst")) {
            instance.inputList.clear();
            instance.outputList.clear();
            if (!QueryUtil.table_exist(database_name, "dy_pointlist")) {
                return false;
            }
            JSONObject Criteria = new JSONObject();
            Criteria.put("building", building);
            JSONArray content = QueryUtil.select(database_name, "dy_pointlist", Criteria);
            for (int i = 0; i < content.size(); i++) {
                JSONObject json = (JSONObject) content.get(i);
                Entity_point detail = (Entity_point) FastJsonUtil.To_JavaObject(json, Entity_point.class);
                if (!"Inst".equals(detail.data_type)) {
                    continue;
                }
                if ((ConstantCE.funcid_IncludesMap.size() > 0 && !ConstantCE.funcid_IncludesMap.containsKey(detail.funcid))
                        || ConstantCE.funcid_ExcludesMap.containsKey(detail.funcid)) {
                    continue;
                }
                {
                    Entity_ci_instanceinput input = new Entity_ci_instanceinput();
                    input.setObject_sign(object_sign);
                    input.setInstance_name(instance_name);
                    input.setInput_sign("meterdata_" + detail.meter + "_" + detail.funcid);
                    input.setTime_period(detail.collect_cycle);
                    input.setData_class("near");
                    input.setDao_type("fjd_near");
                    JSONObject dao_key = new JSONObject();
                    dao_key.put("building", building);
                    dao_key.put("meter", detail.meter);
                    dao_key.put("funcid", detail.funcid);
                    input.setDao_key(dao_key.toJSONString());
                    input.setSource_object_sign(building);
                    input.setSource_instance_name("fjd");
                    input.setSource_output_sign(detail.meter + "_" + detail.funcid);
                    CoreInput CoreInput = CoreInstance.Convert_input(input);
                    instance.inputList.add(CoreInput);
                }
                {
                    Entity_ci_instanceoutput output = new Entity_ci_instanceoutput();
                    output.setObject_sign(object_sign);
                    output.setInstance_name(instance_name);
                    output.setOutput_sign("inststat_" + detail.meter + "_" + detail.funcid);
                    output.setTime_begin("19700101000000");
                    output.setTime_end("99990101000000");
                    output.setData_class("inststat");
                    output.setDao_type("inststat");
                    JSONObject dao_key = new JSONObject();
                    dao_key.put("building", building);
                    dao_key.put("meter", detail.meter);
                    dao_key.put("funcid", detail.funcid);
                    output.setDao_key(dao_key.toJSONString());
                    output.setNeed_all_var_data(true);
                    output.setTime_period_intialized(true);
                    output.setTime_period(detail.collect_cycle);
                    output.setMode("expression");
                    output.setExpression("meterdata_" + detail.meter + "_" + detail.funcid);
                    CoreOutput CoreOutput = CoreInstance.Convert_output(output);
                    instance.outputList.add(CoreOutput);
                }
            }
        } else if (instance.instance_name.startsWith("servicedata")) {
            // 构建servicedata及其统计的计算实例
            if (ConstantCE.servicedata_use_survey) {
                // 改造report部分
                JSONObject Criteria = new JSONObject();
                Criteria.put("building", building);
                JSONArray content = new JSONArray();
                {
                    JSONArray contentInner = QueryUtil.select(database_name, "dy_reportpointlist", Criteria);
                    for (int i = 0; i < contentInner.size(); i++) {
                        JSONObject contentItem = (JSONObject) contentInner.get(i);
                        contentItem.put("point_class", "report");
                        content.add(contentItem);
                    }
                }
                for (CoreInput CoreInput : instance.inputList) {
                    boolean is_report = false;
                    for (int i = 0; i < content.size(); i++) {
                        JSONObject contentItem = (JSONObject) content.get(i);
                        String meter = (String) contentItem.get("meter");
                        long funcid = (Long) contentItem.get("funcid");
                        String source_output_sign = meter + "_" + funcid;
                        if (source_output_sign.equals(CoreInput.source_output_sign)) {
                            is_report = true;
                            break;
                        }
                    }
                    if (is_report) {
                        CoreInput.data_class = "report";
                        CoreInput.dao_type = "report";
                        CoreInput.source_instance_name = "report";
                    }
                }
            } else {
                instance.inputList.clear();
                instance.outputList.clear();
                if (!QueryUtil.table_exist(database_name, "dy_pointlist")) {
                    return false;
                }
                JSONObject Criteria = new JSONObject();
                Criteria.put("building", building);
                JSONArray content = new JSONArray();
                {
                    JSONArray contentInner = QueryUtil.select(database_name, "dy_pointlist", Criteria);
                    for (int i = 0; i < contentInner.size(); i++) {
                        JSONObject contentItem = (JSONObject) contentInner.get(i);
                        contentItem.put("point_class", "fjd_near");
                        content.add(contentItem);
                    }
                }
                {
                    JSONArray contentInner = QueryUtil.select(database_name, "dy_reportpointlist", Criteria);
                    for (int i = 0; i < contentInner.size(); i++) {
                        JSONObject contentItem = (JSONObject) contentInner.get(i);
                        contentItem.put("point_class", "report");
                        content.add(contentItem);
                    }
                }
                for (int i = 0; i < content.size(); i++) {
                    JSONObject json = (JSONObject) content.get(i);
                    Entity_point detail = (Entity_point) FastJsonUtil.To_JavaObject(json, Entity_point.class);
                    if (!"Acc".equals(detail.data_type) && !"InAc".equals(detail.data_type)) {
                        continue;
                    }
                    if ((ConstantCE.funcid_IncludesMap.size() > 0 && !ConstantCE.funcid_IncludesMap.containsKey(detail.funcid))
                            || ConstantCE.funcid_ExcludesMap.containsKey(detail.funcid)) {
                        continue;
                    }
                    {
                        Entity_ci_instanceinput input = new Entity_ci_instanceinput();
                        input.setObject_sign(object_sign);
                        input.setInstance_name(instance_name);
                        input.setInput_sign("meterdata_" + detail.meter + "_" + detail.funcid);
                        input.setTime_period(detail.collect_cycle);
                        if ("report".equals(detail.point_class)) {
                            input.setData_class("report");
                            input.setDao_type("report");
                            input.setSource_instance_name("report");
                        } else {
                            input.setData_class("near");
                            input.setDao_type("fjd_near");
                            input.setSource_instance_name("fjd");
                        }
                        input.setSource_object_sign(building);
                        input.setSource_output_sign(detail.meter + "_" + detail.funcid);
                        JSONObject dao_key = new JSONObject();
                        dao_key.put("building", building);
                        dao_key.put("meter", detail.meter);
                        dao_key.put("funcid", detail.funcid);
                        input.setDao_key(dao_key.toJSONString());
                        CoreInput CoreInput = CoreInstance.Convert_input(input);
                        instance.inputList.add(CoreInput);
                    }
                    {
                        Entity_ci_instanceoutput output = new Entity_ci_instanceoutput();
                        output.setObject_sign(object_sign);
                        output.setInstance_name(instance_name);
                        output.setOutput_sign("servicedata_" + detail.meter + "_" + detail.funcid);
                        output.setTime_begin("19700101000000");
                        output.setTime_end("99990101000000");
                        output.setData_class("consumption");
                        output.setDao_type("servicedata");
                        JSONObject dao_key = new JSONObject();
                        dao_key.put("building", building);
                        dao_key.put("meter", detail.meter);
                        dao_key.put("funcid", detail.funcid);
                        output.setDao_key(dao_key.toJSONString());
                        output.setNeed_all_var_data(true);
                        output.setTime_period_intialized(true);
                        output.setTime_period(detail.collect_cycle);
                        output.setMode("expression");
                        output.setExpression(
                                "meterdata_" + detail.meter + "_" + detail.funcid + "'+1-" + "meterdata_" + detail.meter + "_" + detail.funcid);
                        CoreOutput CoreOutput = CoreInstance.Convert_output(output);
                        instance.outputList.add(CoreOutput);
                    }
                }
            } // end if

            // servicedata及其统计中需要data_type
            JSONObject Criteria = new JSONObject();
            Criteria.put("building", building);
            JSONArray content = new JSONArray();
            {
                JSONArray contentInner = QueryUtil.select(database_name, "dy_pointlist", Criteria);
                for (int i = 0; i < contentInner.size(); i++) {
                    JSONObject contentItem = (JSONObject) contentInner.get(i);
                    contentItem.put("point_class", "fjd_near");
                    content.add(contentItem);
                }
            }
            {
                JSONArray contentInner = QueryUtil.select(database_name, "dy_reportpointlist", Criteria);
                for (int i = 0; i < contentInner.size(); i++) {
                    JSONObject contentItem = (JSONObject) contentInner.get(i);
                    contentItem.put("point_class", "report");
                    content.add(contentItem);
                }
            }
            for (int i = 0; i < content.size(); i++) {
                JSONObject json = (JSONObject) content.get(i);
                Entity_point detail = (Entity_point) FastJsonUtil.To_JavaObject(json, Entity_point.class);
                if (!"Acc".equals(detail.data_type) && !"InAc".equals(detail.data_type)) {
                    continue;
                }
                if ((ConstantCE.funcid_IncludesMap.size() > 0 && !ConstantCE.funcid_IncludesMap.containsKey(detail.funcid))
                        || ConstantCE.funcid_ExcludesMap.containsKey(detail.funcid)) {
                    continue;
                }
                for (CoreInput CoreInput : instance.inputList) {
                    if (CoreInput.input_sign.equals("meterdata_" + detail.meter + "_" + detail.funcid)) {
                        CoreInput.data_type = detail.data_type;
                        break;
                    }
                }
            }
        } else if (instance.instance_name.startsWith("tenant")) {
            instance.inputList.clear();
            instance.outputList.clear();
            List<String> data_typeList = new ArrayList<String>();
            {
                data_typeList.add("Acc");
            }
            if (!QueryUtil.table_exist(database_name, "dy_pointlist")) {
                return false;
            }
            JSONObject Criteria = new JSONObject();
            Criteria.put("building", building);
            JSONArray content = QueryUtil.select(database_name, "dy_pointlist", Criteria);
            Map<String, Entity_point> pointMap = new HashMap<String, Entity_point>();
            for (int i = 0; i < content.size(); i++) {
                JSONObject json = (JSONObject) content.get(i);
                Entity_point detail = (Entity_point) FastJsonUtil.To_JavaObject(json, Entity_point.class);
                boolean match = false;
                for (String data_type : data_typeList) {
                    if (detail.data_type.equalsIgnoreCase(data_type)) {
                        match = true;
                        break;
                    }
                }
                if (!match) {
                    continue;
                }
                if ((ConstantCE.funcid_IncludesMap.size() > 0 && !ConstantCE.funcid_IncludesMap.containsKey(detail.funcid))
                        || ConstantCE.funcid_ExcludesMap.containsKey(detail.funcid)) {
                    continue;
                }
                pointMap.put(detail.meter + "_" + detail.funcid, detail);
            }

            TenantUtil.buildInstance(database_name, Criteria, instance, pointMap);
            if ("tenant_max".equals(instance.instance_name)) {
                instance = SurveyUtil.Convert_max(instance);
            }
        } // end if

        if (instance.instance_name.endsWith("_stat")) {
            instance.inputList.clear();
            for (CoreOutput output : instance.outputList) {
                Entity_ci_instanceinput input = new Entity_ci_instanceinput();
                input.setObject_sign(object_sign);
                input.setInstance_name(instance_name);
                input.setInput_sign(output.output_sign);
                if ("stat_Inst_stat".equals(instance.instance_name)) {
                    List<String> biggerList = Parser_time_period.getBigger("1y", output.time_period);
                    input.setTime_period(biggerList.get(1));
                } else {
                    input.setTime_period("1d");
                }
                input.setData_class(output.data_class);
                input.setDao_type(output.dao_type);
                input.setSource_object_sign(object_sign);
                input.setSource_instance_name(instance_name.substring(0, instance_name.length() - "_stat".length()));
                input.setSource_output_sign(output.output_sign);
                input.setDao_key(output.dao_key.toJSONString());
                CoreInput CoreInput = CoreInstance.Convert_input(input);
                instance.inputList.add(CoreInput);

                output.expression = output.output_sign;
                output.dao_type = output.dao_type + "_stat";
            }
        }

        CoreComputeUtil.Refresh(instance);
        boolean computed = ComputeEngineUtil.ComputeAndSave(database_name, instance, dao_log);

        return computed;
    }
}

class InstanceComparator implements Comparator<String> {
    @Override
    public int compare(String arg1, String arg2) {
        int index1 = arg1.indexOf('_');
        int index2 = arg2.indexOf('_');
        String object_sign1 = arg1.substring(0, index1);
        String instance_name1 = arg1.substring(index1 + 1);
        String object_sign2 = arg2.substring(0, index2);
        String instance_name2 = arg2.substring(index2 + 1);
        int cmp_ = object_sign1.compareTo(object_sign2);
        if (cmp_ == 0) {
            int cmp__ = instance_name1.compareTo(instance_name2);
            return cmp__;
        } else {
            return cmp_;
        }
    }
}