package com.sagacloud.util.compute_engine.fjd;

import com.sagacloud.util.compute_engine.core.CoreComputeUtil;
import com.sagacloud.util.compute_engine.util.TimeSpan;
import lombok.extern.slf4j.Slf4j;

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

@Slf4j
public class ThreadComputeJob extends Thread {
    private volatile boolean stop = false;

    public ThreadComputeJob() {
    }

    public void requestStop() {
        stop = true;
    }

    @Override
    public void run() {
        while (!stop) {
            try {
                boolean worked = this.Process();
                if (!worked) {
                    try {
                        Thread.sleep(1000L);
                    } catch (InterruptedException e1) {
                        log.error(e1.getMessage(),e1);
                        
                    }
                }
            } catch (Exception e) {
                log.error(e.getMessage(),e);
            }

        }
    }

    private boolean Process() throws Exception {
        ComputeJobItem ComputeJobItem = ComputeJobRepository.Instance().Pop();
        if (ComputeJobItem == null) {
            return false;
        }

        try {
            Map<String, List<TimeSpan>> outputTimeSpanMap = new HashMap<String, List<TimeSpan>>();
            if (ComputeJobItem.instance_name.startsWith("fjd_0")) {
                outputTimeSpanMap = Compute_fjd_0.Compute(ComputeJobItem.object_sign, ComputeJobItem.instance_name, ComputeJobItem.inputList,
                        ComputeJobItem.outputList, ComputeJobItem.compute_round, ComputeJobItem.inputdetailListMap,
                        ComputeJobItem.outputdetailListMap, ComputeJobItem.compute_time_from, ComputeJobItem.compute_time_to, ComputeJobItem.dao_log);
            } else if (ComputeJobItem.instance_name.startsWith("fjd_1")) {
                outputTimeSpanMap = Compute_fjd_1.Compute(ComputeJobItem.object_sign, ComputeJobItem.instance_name, ComputeJobItem.inputList,
                        ComputeJobItem.outputList, ComputeJobItem.compute_round, ComputeJobItem.inputdetailListMap,
                        ComputeJobItem.outputdetailListMap, ComputeJobItem.dao_log);
            } else if ("stat_Inst".equals(ComputeJobItem.instance_name)) {
                outputTimeSpanMap = Compute_stat_Inst.Compute(ComputeJobItem.object_sign, ComputeJobItem.instance_name, ComputeJobItem.inputList,
                        ComputeJobItem.outputList, ComputeJobItem.compute_round, ComputeJobItem.inputdetailListMap,
                        ComputeJobItem.outputdetailListMap, ComputeJobItem.dao_log);
            } else if ("servicedata".equals(ComputeJobItem.instance_name) || "servicedataoftenant".equals(ComputeJobItem.instance_name)) {
                outputTimeSpanMap = Compute_servicedata.Compute(ComputeJobItem.object_sign, ComputeJobItem.instance_name, ComputeJobItem.inputList,
                        ComputeJobItem.outputList, ComputeJobItem.compute_round, ComputeJobItem.inputdetailListMap,
                        ComputeJobItem.outputdetailListMap, ComputeJobItem.dao_log);
            } else {
                outputTimeSpanMap = CoreComputeUtil.ComputeOnce(ComputeJobItem.object_sign, ComputeJobItem.instance_name, ComputeJobItem.inputList,
                        ComputeJobItem.outputList, ComputeJobItem.compute_round, ComputeJobItem.inputdetailListMap,
                        ComputeJobItem.outputdetailListMap, ComputeJobItem.dao_log, true);
            }
            Iterator<String> iter = outputTimeSpanMap.keySet().iterator();
            while (iter.hasNext()) {
                String key = iter.next();
                List<TimeSpan> value = outputTimeSpanMap.get(key);
                ComputeJobItem.outputTimeSpanMap.put(key, value);
            }
            ComputeJobRepository.Instance().Finish(ComputeJobItem);
        } catch (Exception e) {
            log.warn(e.getMessage(), e);
            ComputeJobRepository.Instance().FinishWithError(ComputeJobItem);
        }

        return true;
    }
}
