package com.sagacloud.util.compute_engine.fjd;

import cn.hutool.core.thread.ThreadUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.sagacloud.repository.HBaseSaveManagerRepository;
import com.sagacloud.util.compute_engine.util.ConstantCE;
import com.sagacloud.util.compute_engine.util.DateAccelerator;
import com.sagacloud.util.compute_engine.util.QueryUtil;
import com.zillion.database.agent.ZillionAgent;
import com.zillion.util.zk.ZKLockRW;
import com.zillion.util.zk.ZKWrapper;
import lombok.extern.slf4j.Slf4j;
import org.apache.zookeeper.CreateMode;
import org.apache.zookeeper.data.Stat;

import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

@Slf4j
public class ThreadDelayItem extends Thread {
    ZillionAgent agent;
    String database_name;
    List<ThreadCompute> threadList;
    String path_delay = ConstantCE.zk_root + "/" + "delay";
    String path_delay_lock = ConstantCE.zk_root + "/" + "delay_lock";
    SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");
    private volatile boolean stop = false;

    public ThreadDelayItem(String database_name) {

        this.database_name = database_name;
    }

    public void requestStop() {
        stop = true;
    }

    @Override
    public void run() {
        ZKWrapper ZK = HBaseSaveManagerRepository.randomZillionAgent().container.ZK;
        Date last_read_zk = null;
        List<String> children = null;
        int index_child = 0;
        try {
            children = ZK.getChildren(path_delay, false);
            last_read_zk = DateAccelerator.Get();
        } catch (Exception e) {
           log.error(e.getMessage(),e);
        }
        while (!stop) {
            ThreadUtil.safeSleep(1);
            try {
                Date curr = DateAccelerator.Get();
                if (children == null || last_read_zk == null || curr.getTime() - last_read_zk.getTime() > 1000L * 60) {
                    children = ZK.getChildren(path_delay, false);
                    last_read_zk = DateAccelerator.Get();
                }
            } catch (Exception e) {
                log.error(e.getMessage(), e);
            }

            if (children != null && children.size() > 0) {
                if (index_child >= children.size()) {
                    index_child = 0;
                }

                String child = children.get(index_child);
                try {
                    this.Process(ZK, child);
                } catch (Exception e) {
                    log.error(e.getMessage(), e);
                }

                index_child++;
            }
        }
    }

    private void Process(ZKWrapper ZK, String building) throws Exception {
        String parent_path_meta = path_delay_lock + "/" + building;
        Stat stat = ZK.exists(parent_path_meta, false);
        if (stat == null) {
            try {
                ZK.create(parent_path_meta, "true".getBytes(), CreateMode.PERSISTENT);
            } catch (Exception e) {
               log.error(e.getMessage(),e);
            }
        }
        String lock_type_meta = "w";
        // 获取锁
        String node_path_meta = ZKLockRW.Create(ZK, parent_path_meta, lock_type_meta, "true");
        try {
            boolean wait_success = ZKLockRW.WaitOnce(ZK, parent_path_meta, node_path_meta);
            log.debug("ThreadDelay instance wait lock " + building + " " + node_path_meta + " " + wait_success);
            if (wait_success) {
                this.ProcessInner(ZK, building);
            }
        } catch (Exception e) {
            throw e;
        } finally {
            // 释放锁
            ZKLockRW.Finish(ZK, node_path_meta);
            log.debug("ThreadDelay instance finish lock " + building + " " + node_path_meta);
        }
    }

    private void ProcessInner(ZKWrapper ZK, String building) {
        Calendar calendar = Calendar.getInstance();
        try {
            Stat stat = new Stat();
            String data = new String(ZK.getData(path_delay + "/" + building, false, stat));
            Date processed_time = sdf.parse(data);
            Date curr_proc_time = new Date(DateAccelerator.Get().getTime() / (1000L * 60 * ConstantCE.original_delay_period_minute)
                    * (1000L * 60 * ConstantCE.original_delay_period_minute));
            if (curr_proc_time.getTime() > processed_time.getTime()) {
                log.warn("delaylog" + "\t" + building + "\t" + sdf.format(curr_proc_time) + "\t" + "begin");
                JSONObject Criteria = new JSONObject();
                Criteria.put("building", building);
                JSONArray Content = QueryUtil.select(database_name, "original_present", Criteria);
                JSONArray InsertObjects = new JSONArray();
                long good_count = 0L;
                long delay_count = 0L;
                for (int i = 0; i < Content.size(); i++) {
                    JSONObject item = (JSONObject) Content.get(i);
                    String meter = (String) item.get("meter");
                    long funcid = (Long) item.get("funcid");
                    String receivetime = (String) item.get("receivetime");
                    if (curr_proc_time.getTime() - sdf.parse(receivetime).getTime() <= 1000L * 60 * ConstantCE.original_delay_period_minute
                            * ConstantCE.original_delay_delayperiodcount) {
                        JSONObject InsertObject = new JSONObject();
                        InsertObject.put("building", building);
                        InsertObject.put("meter", meter);
                        InsertObject.put("funcid", funcid);
                        InsertObject.put("operate_time", sdf.format(curr_proc_time));
                        InsertObject.put("data_time", receivetime);
                        InsertObject.put("delay_second", (curr_proc_time.getTime() - sdf.parse(receivetime).getTime()) / 1000L);
                        if (curr_proc_time.getTime() - sdf.parse(receivetime).getTime() <= 1000L * 60 * ConstantCE.original_delay_period_minute
                                * ConstantCE.original_delay_goodperiodcount) {
                            good_count++;
                            InsertObject.put("flag", 0L);
                        } else {
                            delay_count++;
                            InsertObject.put("flag", 1L);
                        }
                        InsertObjects.add(InsertObject);
                    }
                }
                if (InsertObjects.size() > 0) {
                    QueryUtil.batch_insert(database_name, "original_delaydetail", InsertObjects);
                }
                {
                    JSONObject InsertObject = new JSONObject();
                    InsertObject.put("building", building);
                    InsertObject.put("operate_time", sdf.format(curr_proc_time));
                    InsertObject.put("total_count", Content.size());
                    InsertObject.put("good_count", good_count);
                    InsertObject.put("delay_count", delay_count);
                    QueryUtil.insert(database_name, "original_delaylog", InsertObject);
                }

                ZK.setData(path_delay + "/" + building, sdf.format(curr_proc_time).getBytes(), -1);
                log.warn("delaylog" + "\t" + building + "\t" + sdf.format(curr_proc_time) + "\t" + "end");

                calendar.setTime(curr_proc_time);
                if (calendar.get(Calendar.HOUR_OF_DAY) == 0 && calendar.get(Calendar.MINUTE) == 0 && calendar.get(Calendar.SECOND) == 0) {
                    JSONObject job = new JSONObject();
                    job.put("building", building);
                    job.put("curr_proc_time", sdf.format(curr_proc_time));
                    ThreadDelay.Push(job);
                }
            }
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
    }
}