package com.sagacloud.util.compute_engine.fjd;

import cn.hutool.core.thread.ThreadUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.sagacloud.util.compute_engine.util.ConstantCE;
import com.sagacloud.util.compute_engine.util.QueryUtil;
import com.zillion.database.agent.ZillionAgent;
import lombok.extern.slf4j.Slf4j;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

@Slf4j
public class ThreadDelayStat extends Thread {
    ZillionAgent agent;
    String database_name;
    List<ThreadCompute> threadList;
    String path_delay = ConstantCE.zk_root + "/" + "delay";
    String path_delay_lock = ConstantCE.zk_root + "/" + "delay_lock";
    SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");
    private volatile boolean stop = false;

    public ThreadDelayStat(String database_name) {

        this.database_name = database_name;
    }

    public void requestStop() {
        stop = true;
    }

    @Override
    public void run() {
        while (!stop) {
            ThreadUtil.safeSleep(1);
            JSONObject job = ThreadDelay.Pop();
            if (job != null) {
                try {
                    this.stat(job);
                } catch (Exception e) {
                    log.warn(e.getMessage(), e);
                }
            }
        }
    }

    private void stat(JSONObject job) throws Exception {
        String building = (String) job.get("building");
        Date curr_proc_time = sdf.parse((String) job.get("curr_proc_time"));
        Date last_time = new Date(curr_proc_time.getTime() - 1000L * 60 * 60 * 24);
        log.warn("delaystat" + "\t" + building + "\t" + sdf.format(last_time) + "\t" + "begin");
        JSONObject Criteria = new JSONObject();
        Criteria.put("building", building);
        JSONArray Content = QueryUtil.select(database_name, "original_present", Criteria);
        for (int i = 0; i < Content.size(); i++) {
            JSONObject item = (JSONObject) Content.get(i);
            String meter = (String) item.get("meter");
            long funcid = (Long) item.get("funcid");
            String receivetime = (String) item.get("receivetime");
            if (last_time.getTime() - sdf.parse(receivetime).getTime() > 1000L * 60 * ConstantCE.original_delay_period_minute
                    * ConstantCE.original_delay_delayperiodcount) {
                continue;
            }

            JSONObject CriteriaInner = new JSONObject();
            CriteriaInner.put("building", building);
            CriteriaInner.put("meter", meter);
            CriteriaInner.put("funcid", funcid);
            JSONObject Range = new JSONObject();
            Range.put("$gte", sdf.format(last_time));
            Range.put("$lt", sdf.format(curr_proc_time));
            CriteriaInner.put("operate_time", Range);
            JSONArray ContentInner = QueryUtil.select(database_name, "original_delaydetail", CriteriaInner);
            if (ContentInner.size() > 0) {
                long valid_count = 0;
                long delay_second_total = 0;
                Long delay_second_max = null;
                Long delay_second_min = null;
                for (int ii = 0; ii < ContentInner.size(); ii++) {
                    JSONObject itemInner = (JSONObject) ContentInner.get(ii);
                    valid_count++;
                    long delay_second = (Long) itemInner.get("delay_second");
                    delay_second_total += delay_second;
                    if (delay_second_max == null || delay_second_max < delay_second) {
                        delay_second_max = delay_second;
                    }
                    if (delay_second_min == null || delay_second_min > delay_second) {
                        delay_second_min = delay_second;
                    }
                }

                JSONObject InsertObject = new JSONObject();
                InsertObject.put("building", building);
                InsertObject.put("meter", meter);
                InsertObject.put("funcid", funcid);
                InsertObject.put("operate_time", sdf.format(last_time));
                InsertObject.put("valid_count", valid_count);
                InsertObject.put("delay_second_avg", 1.0 * delay_second_total / ContentInner.size());
                InsertObject.put("delay_second_max", delay_second_max);
                InsertObject.put("delay_second_min", delay_second_min);
                QueryUtil.insert(database_name, "original_delaystat", InsertObject);
            }
        }
        log.warn("delaystat" + "\t" + building + "\t" + sdf.format(last_time) + "\t" + "end");
    }
}