package com.sagacloud.util.split;

import com.sagacloud.util.compute_engine.util.DoubleTimeObject;

import java.util.Calendar;
import java.util.List;

public class HistoryUtil {
    private static final HistoryUtil instance = new HistoryUtil();

    private HistoryUtil() {

    }

    public static HistoryUtil Instance() {
        return instance;
    }

    public void GetHistory(List<DoubleTimeObject> serviceDataList, List<Double> estimateList, List<Double> uncertaintyList) {
        Calendar calendar = Calendar.getInstance();
        double[] sumOfValue = new double[168];
        double[] averageOfValue = new double[168];
        double[] sumOfSquares = new double[168];
        int[] count = new int[168];
        // 记录样本是否在拆分中使用次数
        int[] estimateUsed = new int[168];
        // 初始化
        for (int i = 0; i < 168; i++) {
            sumOfValue[i] = 0.0;
            averageOfValue[i] = 0.0;
            sumOfSquares[i] = 0.0;
            count[i] = 0;
            estimateUsed[i] = 0;
        }
        // 求总和及个数
        for (int i = 0; i < serviceDataList.size(); i++) {
            DoubleTimeObject objectDataPtr = serviceDataList.get(i);

            calendar.setTime(objectDataPtr.data_time);
            int day_of_week = calendar.get(Calendar.DAY_OF_WEEK);
            int hour_of_day = calendar.get(Calendar.HOUR_OF_DAY);
            int indexHour = (day_of_week - 1) * 24 + hour_of_day;

            sumOfValue[indexHour] += objectDataPtr.data_value;
            count[indexHour]++;
        }
        // 求平均值
        for (int i = 0; i < 168; i++) {
            if (count[i] > 0) {
                averageOfValue[i] = sumOfValue[i] / count[i];
            }
        }
        // 求平方和
        for (int i = 0; i < serviceDataList.size(); i++) {
            DoubleTimeObject objectDataPtr = serviceDataList.get(i);

            calendar.setTime(objectDataPtr.data_time);
            int day_of_week = calendar.get(Calendar.DAY_OF_WEEK);
            int hour_of_day = calendar.get(Calendar.HOUR_OF_DAY);
            int indexHour = (day_of_week - 1) * 24 + hour_of_day;

            double pingfang = Math.pow(objectDataPtr.data_value - averageOfValue[indexHour], 2);
            sumOfSquares[indexHour] += pingfang;
        }
        // 求估计值和不确定度
        for (int i = 0; i < 168; i++) {
            if (count[i] > 0) {
                estimateList.add(averageOfValue[i]);
                uncertaintyList.add(Math.sqrt(sumOfSquares[i] / count[i]));
            } else {
                estimateList.add(null);
                uncertaintyList.add(null);
            }
        }
    }
}
